library(MASS)

credit<-read.csv('/Users/Thierry/Documents/R/Data/Economics/default_credit_card.csv',
               sep=";",header=TRUE)
credit$X2<-as.factor(credit$X2)
levels(credit$X2)<-c("M","F")
credit$X3<-as.factor(credit$X3)
credit$X4<-as.factor(credit$X3)
credit$Y<-as.factor(credit$Y)

n<-nrow(credit)
ntrain<-20000
ntest<-n-ntrain
train<-sample(n,ntrain)
credit.train<-credit[train,]
credit.test<-credit[-train,]

# LDA
fit.lda<-lda(Y~.,data=credit.train[,-c(2,3,4)])
pred.lda<-predict(fit.lda,newdata=credit.test[,-c(2,3,4)])
perf.lda<-table(pred.lda$class,credit.test$Y)
err.lda<-1-sum(diag(perf.lda))/ntest
print(err.lda)

# QDA
fit.qda<-qda(Y~.,data=credit.train[,-c(2,3,4)])
pred.qda<-predict(fit.qda,newdata=credit.test[,-c(2,3,4)])
perf.qda<-table(pred.qda$class,credit.test$Y)
err.qda<-1-sum(diag(perf.qda))/ntest
print(err.qda)

# Naive Bayes
library(naivebayes)
fit.naive<-naive_bayes(Y~.,data=credit.train)
pred.naive<-predict(fit.naive,newdata=credit.test)
perf.naive<-table(pred.naive,credit.test$Y)
err.naive<-1-sum(diag(perf.naive))/ntest
print(err.naive)

# Logistic regression

fit.logreg<- glm(Y~.,data=credit.train,family=binomial)
pred.logreg<-predict(fit.logreg,newdata=credit.test,type='response')
perf <-table(credit.test$Y,pred.logreg>0.5)
print(perf)
err.logreg <-1-sum(diag(perf))/ntest  # error rate
print(err.logreg)

print(c(err.lda,err.qda,err.naive,err.logreg))


# Plot of ROC curves
library(pROC)

roc_lda<-roc(credit.test$Y,as.vector(pred.lda$x))
plot(roc_lda)

roc_qda<-roc(credit.test$Y,as.vector(pred.qda$posterior[,1]))
plot(roc_qda,add=TRUE,col='red')

pred.nb.prob<-predict(fit.naive,newdata=credit.test,type="prob")
roc_nb<-roc(credit.test$Y,as.vector(pred.nb.prob[,1]))
plot(roc_nb,add=TRUE,col='blue')

roc_logreg<-roc(credit.test$Y,as.vector(pred.logreg))
plot(roc_logreg,add=TRUE,col='green')


#----- Exercise 2

library(mvtnorm)
mu1<-c(0,0)
mu2<-c(0,2)
mu3<-c(2,0)
Sigma<-matrix(c(1,0.5,0.5,2),2,2)
p1<-0.3
p2<-0.3
p3<-1-p1-p2

# Q1

# Function to generate a data set
gen.data<- function(N,mu1,mu2,mu3,Sigma1,Sigma2,Sigma3,p1,p2){
  y<-sample(3,N,prob=c(p1,p2,1-p1-p2),replace=TRUE)
  X<-matrix(0,N,2)
  N1<-length(which(y==1)) # number of objects from class 1
  N2<-length(which(y==2))
  N3<-length(which(y==3))
  X[y==1,]<-rmvnorm(N1,mu1,Sigma1)
  X[y==2,]<-rmvnorm(N2,mu2,Sigma2)
  X[y==3,]<-rmvnorm(N3,mu3,Sigma3)
  return(list(X=X,y=y))
}


# Test set generation
test<-gen.data(N=10000,mu1,mu2,mu3,Sigma,Sigma,Sigma,
               p1=p1,p2=p2)
plot(test$X,col=test$y,pch=".")

# Test set classification using Bayes rule

g<-cbind(p1*dmvnorm(test$X,mu1,Sigma),
         p2*dmvnorm(test$X,mu2,Sigma),
         p3*dmvnorm(test$X,mu3,Sigma))
ypred<-max.col(g)
table(test$y,ypred)
errb<-mean(test$y != ypred)
print(errb)

# Q2 (LDA)

train<-gen.data(N=50,mu1,mu2,mu3,Sigma,Sigma,Sigma,
                p1=p1,p2=p2)
train.frame<-data.frame(train)
test.frame<-data.frame(test)

fit<- lda(y~.,data=train.frame)
pred<-predict(fit,newdata=test.frame)
err<-mean(test$y != pred$class)
print(c(errb,err))